// CSineGenerateCP.c MEX file generate sinusoids with centre phase
#include "mex.h"
#include <math.h>
#define nsp_UsesAll	// uses Intel library functions
#include "nsp.h"
/* this is a mex function specifically designed for use with reSynth.m */
/* error trapping is kept to a minimum to enhance speed */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	double *f,*amp,*df,*da,*N,*centrePhase,*Fs,*meanMult,*tone,*endAmp,*endPhi,*tonePhases,*toneAmps;
	double temp,ampExponent,recipTL_1,recip2TL_1,dPi_Fs;
	int toneLength,mid,n;
	// variables passed from MATLAB
	f = mxGetPr(prhs[0]);
	amp = mxGetPr(prhs[1]);
	df = mxGetPr(prhs[2]);
	da = mxGetPr(prhs[3]);
	N = mxGetPr(prhs[4]);
	centrePhase = mxGetPr(prhs[5]);
	Fs = mxGetPr(prhs[6]);
	meanMult = mxGetPr(prhs[7]);

	toneLength = (int)N[0];
	recipTL_1 = 1/(N[0]-1);
	recip2TL_1 = 2/(N[0]-1);
	dPi_Fs = NSP_2PI/Fs[0];

	// variables passed back to MATLAB
	plhs[0] = mxCreateDoubleMatrix(1,toneLength,mxREAL);
	plhs[1] = mxCreateDoubleMatrix(1,1,mxREAL);
	plhs[2] = mxCreateDoubleMatrix(1,1,mxREAL);
	tone = mxGetPr(plhs[0]);
	endAmp = mxGetPr(plhs[1]);
	endPhi = mxGetPr(plhs[2]);

	// temporary variables
	tonePhases = (double *) mxMalloc(toneLength*sizeof(double));
	toneAmps = (double *) mxMalloc(toneLength*sizeof(double));
	
	// toneLength is length of output sinusoid in samples
	// generate phases
	mid = toneLength/2;
	ampExponent = 0.025*da[0];
	//float TTS = (float)toneTableSize;
	tonePhases[mid] = *centrePhase;	
	toneAmps[mid] = 1;
	//temp = tonePhases[mid]* recip2pi * TTS;
	tone[mid] = toneAmps[mid]*sin(tonePhases[mid]);
	// two loops needed for middle of frame phase value
	for (n = mid+1; n < toneLength; n++)	{
		tonePhases[n] = tonePhases[n-1] + ((f[0]+(df[0]*((((double)n-1)*recipTL_1)-0.5)))*dPi_Fs);
		toneAmps[n] = pow(10,((double)n-(double)mid)*recip2TL_1*ampExponent);
		// sin function synthesis
		tone[n] = toneAmps[n] * sin(tonePhases[n]);
		// wavetable lookup
		//if(tonePhases[n]>NSP_2PI)
		//	tonePhases[n] -= NSP_2PI;
		//temp = tonePhases[n]* recip2pi * TTS;
		//tone[n] = toneAmps[n]*toneTable[(int)temp];
	}
	ampExponent *= -1.0;
	for (n = mid-1; n >= 0; n--)	{
		tonePhases[n] = tonePhases[n+1] - ((f[0]-(df[0]*(0.5-(((double)n-1)*recipTL_1))))*dPi_Fs);
		toneAmps[n] = pow(10,((double)mid-(double)n)*recip2TL_1*ampExponent);
		// sin function synthesis
		tone[n] = toneAmps[n] * sin(tonePhases[n]);
		// wavetable lookup
		//if(tonePhases[n]<0)
		//	tonePhases[n] += NSP_2PI;
		//temp = tonePhases[n]* recip2pi * TTS;
		//tone[n] = toneAmps[n]*toneTable[(int)temp];
	}
	// now adjust amplitudes to give correct mean amplitude
	temp = nspdMean(toneAmps,toneLength);
	temp = amp[0]/temp;
	nspdbMpy1(temp,tone,toneLength);

	// pass back to MATLAB
	*endAmp = toneAmps[toneLength-1]*temp;
	*endPhi = tonePhases[toneLength-1];
}